@@ -592,29 +592,27 @@ static void pl_shader_tests(pl_gpu gpu)
     }
 
     // Repeat this a few times to test the caching
+    pl_cache cache = pl_cache_create(pl_cache_params( .log = gpu->log ));
+    pl_gpu_set_cache(gpu, cache);
     for (int i = 0; i < 10; i++) {
         if (i == 5) {
             printf("Recreating pl_dispatch to test the caching\n");
             size_t size = pl_dispatch_save(dp, NULL);
             REQUIRE(size);
-            uint8_t *cache = malloc(size);
-            REQUIRE(cache);
-            REQUIRE_CMP(pl_dispatch_save(dp, cache), ==, size, "zu");
+            uint8_t *cache_data = malloc(size);
+            REQUIRE(cache_data);
+            REQUIRE_CMP(pl_dispatch_save(dp, cache_data), ==, size, "zu");
 
             pl_dispatch_destroy(&dp);
             dp = pl_dispatch_create(gpu->log, gpu);
-            pl_dispatch_load(dp, cache);
+            pl_dispatch_load(dp, cache_data);
 
-#ifndef MSAN
-            // Test to make sure the pass regenerates the same cache, but skip
-            // this on MSAN because it doesn't like it when we read from
-            // program cache data generated by the non-instrumented GPU driver
-            uint64_t hash = pl_str_hash((pl_str) { cache, size });
+            // Test to make sure the pass regenerates the same cache
+            uint64_t hash = pl_str_hash((pl_str) { cache_data, size });
             REQUIRE_CMP(pl_dispatch_save(dp, NULL), ==, size, "zu");
-            REQUIRE_CMP(pl_dispatch_save(dp, cache), ==, size, "zu");
-            REQUIRE_CMP(pl_str_hash((pl_str) { cache, size }), ==, hash, PRIu64);
-#endif
-            free(cache);
+            REQUIRE_CMP(pl_dispatch_save(dp, cache_data), ==, size, "zu");
+            REQUIRE_CMP(pl_str_hash((pl_str) { cache_data, size }), ==, hash, PRIu64);
+            free(cache_data);
         }
 
         sh = pl_dispatch_begin(dp);
@@ -638,6 +636,9 @@ static void pl_shader_tests(pl_gpu gpu)
         TEST_FBO_PATTERN(1e-6, "deband iter %d", i);
     }
 
+    pl_gpu_set_cache(gpu, NULL);
+    pl_cache_destroy(&cache);
+
     // Test peak detection and readback if possible
     sh = pl_dispatch_begin(dp);
     pl_shader_sample_nearest(sh, pl_sample_src( .tex = src ));
@@ -949,9 +950,21 @@ static const char *user_shader_tests[] = {
     "//!TYPE DEFINE                                                         \n"
     "100                                                                    \n"
     "                                                                       \n"
+    "//!PARAM testenum                                                      \n"
+    "//!TYPE ENUM DEFINE                                                    \n"
+    "FOO                                                                    \n"
+    "BAR                                                                    \n"
+    "                                                                       \n"
     "//!HOOK MAIN                                                           \n"
     "//!WHEN testconst 30 >                                                 \n"
-    "#error should not be run                                               \n",
+    "#error should not be run                                               \n"
+    "                                                                       \n"
+    "//!HOOK MAIN                                                           \n"
+    "//!WHEN testenum FOO =                                                 \n"
+    "#if testenum == BAR                                                    \n"
+    " #error bad                                                            \n"
+    "#endif                                                                 \n"
+    "vec4 hook() { return vec4(0.0); }                                      \n"
 };
 
 static const char *compute_shader_tests[] = {
@@ -1178,6 +1191,11 @@ static void pl_render_tests(pl_gpu gpu)
     REQUIRE(pl_renderer_get_errors(rr).errors == PL_RENDER_ERR_NONE);
     params = pl_render_default_params;
 
+    struct pl_frame inferred_image = image, inferred_target = target;
+    pl_frames_infer(rr, &inferred_image, &inferred_target);
+    REQUIRE(pl_render_image(rr, &inferred_image, &inferred_target, &params));
+    REQUIRE(pl_renderer_get_errors(rr).errors == PL_RENDER_ERR_NONE);
+
     // Test background blending and alpha transparency
     params.blend_against_tiles = true;
     params.corner_rounding = 0.25f;
@@ -1358,6 +1376,20 @@ static void pl_render_tests(pl_gpu gpu)
     };
     REQUIRE(pl_render_image_mix(rr, &mix, &target, &mix_params));
 
+    // Test inferring frame mix
+    inferred_target = target;
+    pl_frames_infer_mix(rr, &mix, &inferred_target, &inferred_image);
+    REQUIRE(pl_render_image_mix(rr, &mix, &target, &mix_params));
+
+    // Test empty frame mix
+    mix = (struct pl_frame_mix) {0};
+    REQUIRE(pl_render_image_mix(rr, &mix, &target, &mix_params));
+
+    // Test inferring empty frame mix
+    inferred_target = target;
+    pl_frames_infer_mix(rr, &mix, &inferred_target, &inferred_image);
+    REQUIRE(pl_render_image_mix(rr, &mix, &target, &mix_params));
+
     // Test mixer queue
 #define NUM_MIX_FRAMES 20
     const float frame_duration = 1.0 / 24.0;
